"""
	Bobot: Simple Telegram Bot API Wrapper for Python3.5
	https://github.com/zefirka/bobot
"""

from .bobot import init
from .Rule import Rule
from .Parser import Parser
from .Response import Response
